<?php

/**
 * Sample plguins test class
 */
class PlgSampleClass
{
    /**
     * @var    mixed
     * @access private
     */
    var $aParams;

    /**
     * @var    CMS_PluginAPI
     * @access private
     */
    var $api;

    /**
     * @var    DB_si
     * @access private
     */
    var $db;

    /**
     * @var    template
     * @access private
     */
    var $gui;

    /**
     * @var    string
     * @access private
     */
    var $guiBlockName = 'sample_plugin';

    /**
     * @param  string        $side            front|admin
     * @param  mixed         $aPluginParams   $pluginParams variable
     * @param  CMS_PluginAPI $api             $api object
     * @access public
     */
    function PlgSampleClass($side, $aPluginParams, &$api)
    {
        $this->aParams = $aPluginParams;
        $this->api     = &$api;
        $this->db      = &$api->getDB();
        $this->gui     = &$api->getGUI();

        $templateName = $side == 'front' ? 'front' : 'admin';
        $this->gui->addBlock($this->guiBlockName, $this->aParams['templates_path'] . $templateName . '.tpl');
        $this->gui->parseLangFile($this->aParams['templates_path'] . $templateName . '.lng');
    }

    function getHTML()
    {
        $aData = array (
            'plugin_params' => $this->getPluginParamsHTML(),
            'options'       => $this->getUserDefinedOptionsHTML(),
            'database'      => $this->getDbHTML()
        );
        $res = $this->gui->get($this->guiBlockName . ':body', $aData);
        return $res;
    }

    /**
     * Returns HTML table containing plugin parameters
     *
     * @return string
     * @access public
     */
    function getPluginParamsHTML()
    {
        $rows = '';
        foreach ($this->aParams as $key => $value) {
            $rows .= $this->gui->get($this->guiBlockName . ':plugin_params_row', array ('key' => $key, 'value' => $value));
        }
        $res = $this->gui->get($this->guiBlockName . ':plugin_params', array ('rows' => $rows));
        return $res;
    }

    /**
     * Returns HTML table containing user defined plugin options
     *
     * @return string
     * @access public
     * @see    options/options.php
     */
    function getUserDefinedOptionsHTML()
    {
        $rows = '';
        foreach (array (
            'bool', 'uint', 'sint', 'float', 'char', 'text', 'email', 'enum',
            'enum_multi_array', 'enum_multi_array2',
            'date_period', 'date_period_positive', 'date_period_negative'
        ) as $optionNamePostfix) {
            $optionName = 'option_' . $optionNamePostfix;
            $optionValue = (string)$this->api->getOption($optionName);
            $rows .= $this->gui->get($this->guiBlockName . ':options_row', array ('name' => $optionName, 'value' => $optionValue));
        }
        $res = $this->gui->get($this->guiBlockName . ':options', array ('rows' => $rows));
        return $res;
    }

    /**
     * Returns HTML table containing database table data
     *
     * @return string
     * @access public
     */
    function getDbHTML()
    {
        $this->fillDB();

        $rows = '';
        $sql = "SELECT * FROM `cms_plugin_sample`";
        $rs = &$this->db->select($sql);
        while ($aRecord = $rs->nextRecord()) {
            $rows .= $this->gui->get($this->guiBlockName . ':database_row', $aRecord);
        }
        $res = $this->gui->get($this->guiBlockName . ':database', array ('rows' => $rows));
        return $res;
    }

    /**
     * Fills db table with test data if empty
     *
     * @return void
     * @access private
     */
    function fillDB()
    {
        if (!$this->db->getValue("SELECT COUNT(1) FROM `cms_plugin_sample`")) {
            $aRows = array (
                array ('name' => 'record 1', 'value' => 'value 1'),
                array ('name' => 'record 2', 'value' => 'value 2'),
                array ('name' => 'record 3', 'value' => 'value 3'),
            );
            foreach ($aRows as $aRow) {
                $sql = $this->db->genInsertSQL('cms_plugin_sample', $aRow);
                $this->db->execute($sql);
            }
        }
    }
}

